package com.tuoheng.admin.service.accident.query;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tuoheng.admin.entity.Accident;
import com.tuoheng.admin.entity.Dept;
import com.tuoheng.admin.entity.User;
import com.tuoheng.admin.enums.DataPermissionEnum;
import com.tuoheng.admin.enums.FlagEnum;
import com.tuoheng.admin.enums.MarkEnum;
import com.tuoheng.admin.mapper.AccidentMapper;
import com.tuoheng.admin.mapper.DeptMapper;
import com.tuoheng.admin.mapper.UserMapper;
import com.tuoheng.admin.query.AccidentQuery;
import com.tuoheng.admin.utils.CurrentUserUtil;
import com.tuoheng.admin.vo.accident.QueryAccidentPageVO;
import com.tuoheng.common.core.enums.ServiceExceptionEnum;
import com.tuoheng.common.core.exception.ServiceException;
import com.tuoheng.common.core.utils.DateUtils;
import com.tuoheng.common.core.utils.JsonResult;
import com.tuoheng.common.core.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 查询事故卡片分页列表业务层处理
 *
 * @author wanjing
 * @team tuoheng
 * @date 2023-03-03
 */
@Slf4j
@Service
public class QueryAccidentPageListService {

    @Autowired
    private DeptMapper deptMapper;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private AccidentMapper accidentMapper;

    public JsonResult index(AccidentQuery query) {
        //校验
        if (query.getLimit() == null && query.getPage() == null) {
            throw new ServiceException(ServiceExceptionEnum.PARAMETER_IS_NULL);
        }

        User user = CurrentUserUtil.getUserInfo();
        String tenantId = user.getTenantId();

        List<String> deptIdList = this.getDeptIdList(user, query.getDeptId());
        query.setDeptIdList(deptIdList);
        query.setTenantId(tenantId);

        IPage<Accident> page = new Page<>(query.getPage(), query.getLimit());
        IPage<QueryAccidentPageVO> pageData = new Page<>(query.getPage(), query.getLimit());
        //搜索时间
        Date startTime = null;
        Date endTime = null;
        if (StringUtils.isNotEmpty(query.getStartTime()) && StringUtils.isNotEmpty(query.getEndTime())) {
            startTime = DateUtils.dateTime(DateUtils.YYYY_MM_DD_HH_MM_SS, query.getStartTime() + " 00:00:00");
            endTime = DateUtils.dateTime(DateUtils.YYYY_MM_DD_HH_MM_SS, query.getEndTime() + " 23:59:59");
        }
        query.setAccidentStartTime(startTime);
        query.setAccidentEndTime(endTime);
        //获取分页数据
        IPage<Accident> accidentPageData = accidentMapper.selectPage(page, Wrappers.<Accident>lambdaQuery()
                .eq(Accident::getTenantId, tenantId)
                .in(CollectionUtils.isNotEmpty(deptIdList), Accident::getDeptId, deptIdList)
                .eq(null != query.getCode(), Accident::getRoadCode, query.getCode())
                .between(null != query.getAccidentStartTime() && null != query.getAccidentEndTime(),
                        Accident::getCreateTime, query.getAccidentStartTime(), query.getAccidentEndTime())
                .eq(Accident::getFlag, FlagEnum.INSPECTION_ACCIDENT.getCode())
                .eq(Accident::getMark, MarkEnum.VALID.getCode())
                .orderByDesc(Accident::getCreateTime));
        //属性复制
        List<QueryAccidentPageVO> result = accidentPageData.getRecords().stream().map(x -> {
            QueryAccidentPageVO vo = new QueryAccidentPageVO();
            BeanUtils.copyProperties(x, vo);
            //公路代号
            if (StringUtils.isNotEmpty(x.getRoadCode())) {
                vo.setCode(x.getRoadCode());
            }
            //监管部门
            if (StringUtils.isNotEmpty(x.getDeptId())) {
                //根据当前部门id查询对应的部门名称
                Dept dept = deptMapper.selectById(x.getDeptId());
                vo.setName(dept.getName() != null ? dept.getName() : "");
            }

            return vo;
        }).collect(Collectors.toList());
        pageData.setPages(accidentPageData.getPages());
        pageData.setRecords(result);
        pageData.setTotal(accidentPageData.getTotal());
        pageData.setCurrent(accidentPageData.getCurrent());

        return JsonResult.success(pageData);
    }

    /**
     * 根据用户自己的数据权限，查询对应部门的数据
     *
     * @param user
     * @return
     */
    private List<String> getDeptIdList(User user, String deptId) {
        List<String> deptIdList = new ArrayList<>();
        if (StringUtils.isNotEmpty(deptId)) {
            deptIdList.add(deptId);
            return deptIdList;
        }
        if (DataPermissionEnum.ALL.getCode() == user.getDataPermission()) {
            return null;
        } else if (DataPermissionEnum.DEPT_AND_SUB_DEPT.getCode() == user.getDataPermission()) {
            deptIdList = deptMapper.selectAllChildListById(user.getDeptId());
        } else if (DataPermissionEnum.DEPT.getCode() == user.getDataPermission()) {
            deptIdList.add(user.getDeptId());
        }
        return deptIdList;
    }
}
